/****************************************************************************
CREATE DATABASE FUNCTIONS for basic WebCenter16.1 functionality.
****************************************************************************/

/* Run isql -Usa -P"" -e < mssqlserver_functions161.sql > mssqlserver_functions161.log */
/* Run isql -Usa -P"" -S{server} -e < mssqlserver_functions161.sql > mssqlserver_functions161.log */

USE WebCenter4_0
GO

/* Function to check if the user is active based on 'Active_Until' column value. 
 * Returns 1 if the user is active, 0 otherwise.
 */
IF EXISTS (SELECT name FROM sysobjects 
	WHERE name = 'is_user_active' AND type = 'FN')
	DROP FUNCTION webcenter.is_user_active
GO

CREATE function webcenter.is_user_active(@activeUntil DATETIME) 
RETURNS int 
AS 
BEGIN 
	IF (@activeUntil IS NULL) OR @activeUntil > CURRENT_TIMESTAMP
	BEGIN
		return 1;
	END
	return 0;
END
GO


IF EXISTS (SELECT name FROM sysobjects 
	WHERE name = 'figureOutTaskName' AND type = 'FN')
	DROP FUNCTION webcenter.figureOutTaskName
GO

/**
 * Function will either return the next number of the task or "null" when no number can be found.
 */
create function webcenter.figureOutTaskName(@projectID nvarchar(16), @taskName nvarchar(max)) 
returns bigint 
as
begin
	DECLARE @result bigint; 
	DECLARE @length int;

	SET  @length = LEN(@taskName); 

	WITH cte( task_number, rn, maxi ) as (
		SELECT 
			CONVERT(int,SUBSTRING(Task_Name,@length+1,2147483647)),
			row_number() over(order by CONVERT(int,SUBSTRING(Task_Name,@length+1,2147483647))),
			max(CONVERT(int,SUBSTRING(Task_Name,@length+1,2147483647))) over()
		FROM webcenter.PROJECT_WCTASKS 
		WHERE
			Project_ID = @projectID AND 
			(Deleted)=0 AND 
			(UPPER(SUBSTRING(Task_Name,1,@length)))= UPPER(@taskName) AND 
			SUBSTRING(Task_Name,@length+1,2147483647) NOT LIKE '%[^0-9]%'
	) 
	select 
		@result = MIN(
			case 
				when task_number != rn then rn
				when task_number = maxi then task_number +1 
			end
		)   
	from cte;
	return @result
end
GO